@@ -191,13 +191,13 @@ class Parser(object):
                  anchor=YOLOV3_ANCHORS,
                  match_threshold=0.5,
                  unmatched_threshold=0.5,
-                 aug_rand_hflip=False,
+                 aug_rand_hflip=True,
                  aug_scale_min=1.0,
                  aug_scale_max=1.0,
                  use_autoaugment=False,
                  autoaugment_policy_name='v0',
                  autoaugment_ratio=0.8,
-                 skip_crowd_during_training=True,
+                 skip_crowd_during_training=False,
                  max_num_instances=100,
                  use_bfloat16=False,
                  mode=TRAIN):
@@ -329,6 +329,7 @@ class Parser(object):
             # 2  - 预处理
             return self._parse_fn(data)
 
+    # @tf.autograph.experimental.do_not_convert
     def _parse_train_data(self, data):
         """Parses data for training and evaluation."""
         #  1 - 读取基本数据
@@ -360,8 +361,7 @@ class Parser(object):
 
         # 3 - 图片归一化
         # Normalizes image with mean and std pixel values.
-        image = input_utils.normalize_image(image, offset=(0.0, 0.0, 0.0),
-                                            scale=(1., 1., 1.))
+        image = input_utils.normalize_image(image)
 
         # Flips image randomly during training.
         if self._aug_rand_hflip:
